module.exports = function( taskTarget, options ) {
	var grunt = require( "grunt" );
	var touch = require( "touch" );
	var fs = require( "fs" );
	var config = ({
		"wildfly-8.1.0.Final": {
			cwd: "bin",
			deployments: "standalone/deployments",
			startup: {
				success: /WildFly (.*) started in/,
				error: /WildFly started \(with errors\)/
			},
			url: "http://download.jboss.org/wildfly/8.1.0.Final/wildfly-8.1.0.Final.zip",
			// For JBoss servers, it is necessary to create a .dodeploy file to trigger deployment
			beforeStart: function() {
				touch.sync( this.getWARPath() + ".dodeploy" );
			},
			// Remove the .deployed file after the cleanup process
			afterCleanup: function() {
				var deployed = this.getWARPath() + ".deployed";
				var undeployed = this.getWARPath() + ".undeployed";
				if ( fs.existsSync( deployed ) ) {
					grunt.log.debug( "CLEANING " + deployed );
					fs.unlinkSync( deployed );
				}
				if ( fs.existsSync( undeployed ) ) {
					grunt.log.debug( "CLEANING " + undeployed );
					fs.unlinkSync( undeployed );
				}
			}
		}
	})[ options.preset ];
	config.getTaskTargetDir = function() {
		return options.directory + "/" + taskTarget;
	};
	config.getExtractedDir = function() {
		return this.getTaskTargetDir() + "/extracted";
	};
	config.getServerHome = function() {
		return this.getExtractedDir() + "/" + options.preset;
	};
	config.getWARPath = function() {
		return this.getServerHome() + "/" + this.deployments + "/" + options.preset + ".war";
	};
	return config;
};
