module.exports = function() {
	var fs = require( "fs" );
	var grunt = require( "grunt" );
	var http = require( "http" );
	var ProgressBar = require( "progress" );
	var Promise = require( "promise" );
	var url = require( "url" );
	var configs = require( "./configs" );
	return (function( options ) {
		var serverConfig = configs( this.target, options );
		var downloadURL = serverConfig.url;
		var taskTargetDir = serverConfig.getTaskTargetDir();
		var filename = url.parse( downloadURL ).pathname.split( "/" ).pop();
		var downloadedFilePath = taskTargetDir + "/" + filename;
		return new Promise(function( resolve, reject ) {
			var initProgress = function( response ) {
				var template = "Downloading... [:bar] :percent (:currentb/:totalb)";
				var total = +response.headers[ "content-length" ];
				return new ProgressBar( template, {
					complete: "=",
					incomplete: " ",
					width: 20,
					total: total
				});
			};
			var handleResponse = function( response ) {
				var file = fs.createWriteStream( downloadedFilePath );
				var progress = initProgress( response );
				grunt.log.writeln( "Downloading \"" + downloadURL + "\"..." );
				response.on( "data", function( data ) {
					file.write( data );
					progress.tick( data.length );
				})
				.on( "end", function() {
					file.end();
					resolve();
				});
			};
			resolve = resolve.bind( null, downloadedFilePath );
			if ( grunt.file.exists( downloadedFilePath ) ) {
				grunt.log.writeln( "File \"" + downloadedFilePath + "\" already exists." );
				resolve();
				return;
			}
			if ( !grunt.file.isDir( taskTargetDir ) ) {
				grunt.log.writeln( "Creating directory: " + taskTargetDir );
				grunt.file.mkdir( taskTargetDir );
				if ( !grunt.file.isDir( taskTargetDir ) ) {
					reject( new Error( "Failed to create directory: " + taskTargetDir ) );
					return;
				}
			}
			http.get( downloadURL, handleResponse );
		});
	}).bind( this );
};
