module.exports = function() {
	var configs = require( "./configs" );
	var cp = require( "child_process" );
	var extend = require( "node.extend" );
	var grunt = require( "grunt" );
	var os = require( "os" );
	var path = require( "path" );
	var Promise = require( "promise" );
	var registration = require( "./registration" );
	return (function( options ) {
		var reg = registration( this.target );
		var serverConfig = configs( this.target, options );
		var serverHome = serverConfig.getServerHome();
		var fullServerHome = path.resolve( serverHome );
		serverConfig.beforeStart();
		return new Promise(function( resolve, reject ) {
			var serverProcess;
			var listener = function( data ) {
				grunt.log.writeln( data );
				if ( serverConfig.startup.success.test( data ) ) {
					resolve( serverProcess );
					return;
				}
				if ( serverConfig.startup.error.test( data ) ) {
					reg.kill(function() {
						reject( new Error( "JBoss started with errors." ) );
					});
				}
			};
			var startServer = function() {
				var command, args, cwd, spawnOptions = {};
				
				// Define args
				args = options.args;
				if ( Object.prototype.toString.call( args ) == "[object Function]" ) {
					args = args( fullServerHome );
				}
				
				// Platform specific values
				if ( os.platform() === "win32" ) {
					args.unshift( "/c", "standalone" );
					command = process.env.comspec;
				} else if ( os.platform() === "linux" ) {
					command = "sh";
					args.unshift( "standalone.sh" );
				}
				
				// Spawn options
				cwd = path.resolve( fullServerHome + "/" +  serverConfig.cwd );
				spawnOptions.cwd = cwd;
				spawnOptions.env = extend( process.env, {
					JBOSS_HOME: fullServerHome
				});
				
				grunt.log.debug( "Starting server at \"" + spawnOptions.cwd + "\"" );
				grunt.log.debug( "With command: " + command );
				grunt.log.debug( "With arguments: " + JSON.stringify( args ) );
				grunt.log.debug( "With env: " + JSON.stringify( spawnOptions.env ) );
				
				return cp.spawn( command, args, spawnOptions );
			};
			
			serverProcess = startServer();
			reg.register( serverProcess, listener );
		});
	}).bind( this );
};
