"use strict";

module.exports = function( grunt ) {
	var defaults = {
		preset: "wildfly-8.1.0.Final",
		directory: "target",
		args: [],
		keepalive: false
	};
	var execute = {
		server: function() {
			var download = require( "./lib/download" ).call( this );
			var extract = require( "./lib/extract" ).call( this );
			var deploy = require( "./lib/deploy" ).call( this );
			var start = require( "./lib/start" ).call( this );
			var cleanup = require( "./lib/cleanup" ).call( this, grunt );
			
			var taskTarget = this.target;
			var done = this.async();
			var options = this.options( defaults );
			var keepAlive = this.flags.keepalive || options.keepalive;
			
			cleanup( options )
				.then(function() {
					return download( options );
				})
				.then(function( targetFile ) {
					return extract( targetFile, options );
				})
				.then(function() {
					return deploy( options );
				})
				.then(function() {
					return start( options );
				})
				.then(function() {
					if ( !keepAlive ) {
						done();
					}
				})
				.catch(function( err ) {
					grunt.fail.fatal( err );
				});
		},
		kill: function() {
			var cleanup = require( "./lib/cleanup" ).call( this, grunt );
			
			var taskTarget = this.target;
			var done = this.async();
			var options = this.options( defaults );
			
			cleanup( options )
				.then( done )
				.catch(function( err ) {
					grunt.fail.fatal( err );
				});
		}
	};
	grunt.registerMultiTask( "servers", function() {
		if ( this.flags.kill ) {
			execute.kill.call( this );
		} else {
			execute.server.call( this );
		}
	});
};
